/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.filetransfer.listeners;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.logging.Level;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.jbi.messaging.MessagingException;
import javax.xml.transform.stream.StreamSource;

import org.apache.commons.io.FileUtils;
import org.ow2.petals.bc.filetransfer.FileTransferComponent;
import org.ow2.petals.bc.filetransfer.util.FileTransferUtils;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractExternalListener;
import org.ow2.petals.component.framework.util.StringHelper;

import static org.ow2.petals.bc.filetransfer.FileTransferConstants.BACKUP_DIRECTORY;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.ENUM_TRANSFER_MODE_ATTACHMENT;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.ENUM_TRANSFER_MODE_CONTENT;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.POLLING_PERIOD;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.READ_DIRECTORY;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.TRANSFER_MODE;

/**
 * Listen to a directory
 * 
 * @author alouis
 * 
 */
public class FileTransferExternalListener extends AbstractExternalListener {

    private DirectoryPoller poller;

    private File workspace;

    private File working;

    private File backup;

    /**
     * Return the workspace directory
     * 
     * @return
     */
    public File getWorkspaceDirectory() {
        if (this.workspace == null) {
            this.workspace = new File(this.getComponent().getContext().getWorkspaceRoot());
        }
        return this.workspace;
    }

    /**
     * Return the working directory
     * 
     * @return
     * @throws PEtALSCFException
     *      no working directory specified in the SU
     */
    public File getWorkingDirectory() throws IOException {
        if (this.working == null) {
            // retrieve the directory to listen
            String directory = this.getExtensions().get(READ_DIRECTORY);
            directory = FileTransferUtils.evaluateFilePath(directory);
            this.working = new File(directory);
            if (!this.working.exists()) {
                throw new FileNotFoundException(READ_DIRECTORY + " parameter with value '"
                        + directory + "' is invalid");
            }
        }
        return this.working;
    }

    /**
     * Return the backup directory, {@code null} if it is not configured.
     * 
     * @return the backup directory
     */
    public File getBackupDirectory() {
        if (this.backup == null) {
            // retrieve the directory to listen
            String directory = this.getExtensions().get(BACKUP_DIRECTORY);
            if (!StringHelper.isNullOrEmpty(directory)) {
                directory = FileTransferUtils.evaluateFilePath(directory);
                this.backup = new File(directory);
                if (!this.backup.exists()) {
                    this.backup.mkdirs();
                }
            }
        }
        return this.backup;
    }

    @Override
    protected void start() throws PEtALSCDKException {

        // retrieve the polling period
        String s_polling = this.getExtensions().get(POLLING_PERIOD);
        long period = Long.parseLong(s_polling);
        this.getLogger().finest("set polling period to :" + period + "ms");

        // start the poller
        try {
            this.poller = new DirectoryPoller(0, period, this.getWorkingDirectory(), this);
            this.poller.startProcessing();
            this.getLogger().info("start polling directory : " + this.getWorkingDirectory());
        } catch (IOException e) {
            throw new PEtALSCDKException(e);
        }

    }

    @Override
    protected void stop() throws PEtALSCDKException {
        try {
            this.getLogger().info("stop polling directory : " + this.getWorkingDirectory());
        } catch (IOException e) {
            // do nothing
        }
        this.poller.stopProcessing();
    }

    /**
     * Send the content of the file to the JBI bus, then remove it
     * 
     * @param file
     * @throws IOException
     */
    public void processFile(File file) throws IOException {
        this.getLogger().info("process file : " + file.getName());

        // move file in a work or backup dir before processing
        File processedFile = null;
        if (this.getBackupDirectory() == null) {
            processedFile = this.moveFile(file, this.getWorkspaceDirectory());
        } else {
            processedFile = this.moveFile(file, this.getBackupDirectory());
        }

        // process files
        String transferMode = this.getExtensions().get(TRANSFER_MODE);
        this.getLogger().fine("transfer mode : " + transferMode);
        Exchange exchange = null;
        try {
            if (ENUM_TRANSFER_MODE_CONTENT.equalsIgnoreCase(transferMode)) {
                exchange = this.copySource(processedFile);
            } else if (ENUM_TRANSFER_MODE_ATTACHMENT.equalsIgnoreCase(transferMode)) {
                exchange = this.copyAttachment(processedFile, file.getName());
            } else {
                throw new PEtALSCDKException("Transfer mode not recognized :" + transferMode);
            }
            if (this.getBackupDirectory() == null) {
                ((FileTransferComponent) this.getComponent()).addTransferedFile(exchange,
                        processedFile);
            }
        } catch (Exception e) {
            this.getLogger().log(Level.SEVERE, e.getMessage(), e);
            // try to delete the received file if no backup
            if (this.getBackupDirectory() == null) {
                processedFile.delete();
            }
        }

    }

    /**
     * Send the file as a Source
     * 
     * @param file
     * @throws MessagingException
     * @throws FileNotFoundException
     *      file can not be read
     * @throws PEtALSCDKException
     */
    private Exchange copySource(File file) throws MessagingException, FileNotFoundException,
            PEtALSCDKException {
        // create an exchange set with the ServiceUnit values
        Exchange exchange = this.createConsumeExchange();
        StreamSource source = new StreamSource(new FileInputStream(file));
        exchange.setInMessageContent(source);
        this.send(exchange);

        return exchange;
    }

    /**
     * Send the file as an attachment
     * 
     * @param file
     * @throws MessagingException
     * @throws PEtALSCDKException
     */
    private Exchange copyAttachment(File file, String fileName) throws MessagingException,
            PEtALSCDKException {
        // create an exchange set with the ServiceUnit values
        Exchange exchange = this.createConsumeExchange();
        FileDataSource source = new FileDataSource(file);
        DataHandler attachment = new DataHandler(source);
        exchange.getInMessage().addAttachment(fileName, attachment);
        String textString = "<attached-files><file-name>" + fileName
                + "</file-name></attached-files>";
        exchange.setInMessageContent(new StreamSource(new ByteArrayInputStream(textString
                .getBytes())));
        this.send(exchange);

        return exchange;
    }

    /**
     * Move the file to the waste directory
     * 
     * @param file
     *      original file to process
     * @param destDir
     *      working file directory
     * @return the new file reference
     * @throws IOException
     */
    private File moveFile(File file, File destDir) throws IOException {
        String target = "backup-" + this.addDateToFileName(file.getName());
        File newFile = new File(destDir, target);
        FileUtils.copyFile(file, newFile);
        file.delete();
        return newFile;
    }

    /**
     * Add the current time as suffix of the filename
     * 
     * @param filename
     * @return
     */
    private String addDateToFileName(String filename) {
        String result;

        int index = filename.lastIndexOf('.');
        if (index > 0) {
            String prefix = filename.substring(0, filename.lastIndexOf('.'));
            String suffix = filename.substring(filename.lastIndexOf('.'), filename.length());
            result = prefix + System.currentTimeMillis() + suffix;
        } else {
            result = filename + System.currentTimeMillis();
        }

        return result;
    }

}
